@@ -1,5 +1,5 @@
 from abc import ABC, abstractmethod
-from typing import List
+from typing import List, Optional
 
 from ..models.kernel_variable import KernelVariable
 from ..utils import NotebookNode
@@ -16,12 +16,22 @@ class VarsManager(ABC):
     def get_vars_code(self) -> str:
         raise NotImplementedError
 
+    @abstractmethod
+    def get_single_var_code(self, var_name: str) -> str:
+        raise NotImplementedError
+
     @abstractmethod
     def parse_vars_response(
         self, vars_response: NotebookNode
     ) -> List[KernelVariable]:
         raise NotImplementedError
 
+    @abstractmethod
+    def parse_single_var_response(
+        self, var_response: NotebookNode
+    ) -> Optional[KernelVariable]:
+        return None
+
     @property
     def vars(self) -> List[KernelVariable]:
         return self._vars
