@@ -1,4 +1,4 @@
-from typing import List
+from typing import List, Optional
 
 from fastapi.logger import logger
 
@@ -15,6 +15,9 @@ class ZshVarsManager(VarsManager):
     def get_vars_code(self) -> str:
         return raw_get_vars_code
 
+    def get_single_var_code(self, var_name: str) -> str:
+        return ""
+
     def parse_vars_response(
         self, vars_response: NotebookNode
     ) -> List[KernelVariable]:
@@ -37,3 +40,8 @@ class ZshVarsManager(VarsManager):
                 f"{vars_response.text}"
             )
         return vars
+
+    def parse_single_var_response(
+        self, var_response: NotebookNode
+    ) -> Optional[KernelVariable]:
+        return None
