@@ -14,9 +14,10 @@ from jupyter_d1.signals import (CELL_ADDED, CELL_DELETED, CELL_EXECUTION_INPUT,
                                 CELL_EXECUTION_REPLY, CELL_EXECUTION_REQUEST,
                                 CELL_STREAM, CELL_UPDATE, COMPLETE_REPLY,
                                 CONTROL_CHANNEL, HB_CHANNEL, HISTORY_REPLY,
-                                IOPUB_CHANNEL, KERNEL_RESTARTED,
-                                METADATA_UPDATE, PAYLOAD_PAGE, SCRATCH_UPDATE,
-                                SHELL_CHANNEL, STDIN_CHANNEL, VARS_UPDATE)
+                                IOPUB_CHANNEL, KERNEL_INTERRUPTED,
+                                KERNEL_RESTARTED, METADATA_UPDATE,
+                                PAYLOAD_PAGE, SCRATCH_UPDATE, SHELL_CHANNEL,
+                                STDIN_CHANNEL, VARS_UPDATE)
 
 from ..d1_response import D1Encoder, D1Response
 from ..deps import read_access_websocket, write_access, write_access_websocket
@@ -25,7 +26,8 @@ from ..models.cell import (CellAdd, CellKernelMessageWrapper, CellsWrapper,
 from ..models.code_complete import CodeComplete
 from ..models.kernel_message import (KernelMessage, KernelMessagesWrapper,
                                      KernelMessageWrapper)
-from ..models.kernel_variable import KernelVariable, KernelVariablesWrapper
+from ..models.kernel_variable import (KernelVariable, KernelVariablesWrapper,
+                                      KernelVariableWrapper)
 from ..models.notebook import NotebookPath, NotebooksWrapper, NotebookWrapper
 from ..models.scratch import ScratchCode
 from ..settings import settings
@@ -79,6 +81,11 @@ async def get_notebook_cells(uuid: UUID):
     return CellsWrapper(cells=cells)
 
 
+def schedule_notebook_save(background_tasks: BackgroundTasks, uuid: UUID):
+    if manager.get_notebook(uuid).autosave:
+        background_tasks.add_task(manager.save_notebook, uuid=uuid)
+
+
 @router.post(
     "/{uuid}/cells",
     status_code=status.HTTP_201_CREATED,
@@ -103,7 +110,7 @@ async def create_notebook_cell(
         source=cell_add.source,
         before=cell_add.before,
     )
-    background_tasks.add_task(manager.save_notebook, uuid=uuid)
+    schedule_notebook_save(background_tasks, uuid=uuid)
     return CellWrapper(cell=new_cell)
 
 
@@ -124,7 +131,7 @@ async def move_notebook_cell(
     is omitted, the cell will be moved to the end of the cell list.
     """
     manager.move_cell(notebook_id=uuid, cell_id=cell_uuid, before=before)
-    background_tasks.add_task(manager.save_notebook, uuid=uuid)
+    schedule_notebook_save(background_tasks, uuid=uuid)
     return Response(status_code=status.HTTP_204_NO_CONTENT)
 
 
@@ -141,7 +148,7 @@ async def merge_notebook_cells(
 ):
     """Merge a cell with the cell above or below it."""
     manager.merge_cells(notebook_id=uuid, cell_id=cell_uuid, above=above)
-    background_tasks.add_task(manager.save_notebook, uuid=uuid)
+    schedule_notebook_save(background_tasks, uuid=uuid)
     return Response(status_code=status.HTTP_204_NO_CONTENT)
 
 
@@ -160,7 +167,7 @@ async def split_notebook_cell(
     manager.split_cell(
         notebook_id=uuid, cell_id=cell_uuid, split_location=split_location
     )
-    background_tasks.add_task(manager.save_notebook, uuid=uuid)
+    schedule_notebook_save(background_tasks, uuid=uuid)
     return Response(status_code=status.HTTP_204_NO_CONTENT)
 
 
@@ -174,7 +181,7 @@ async def undo_notebook_operation(
 ):
     "Undo the last operaton on the notebook with given UUID."
     manager.undo(uuid)
-    background_tasks.add_task(manager.save_notebook, uuid=uuid)
+    schedule_notebook_save(background_tasks, uuid=uuid)
     return Response(status_code=status.HTTP_204_NO_CONTENT)
 
 
@@ -188,7 +195,7 @@ async def redo_notebook_operation(
 ):
     "Redo the last operaton on the notebook with given UUID."
     manager.redo(uuid)
-    background_tasks.add_task(manager.save_notebook, uuid=uuid)
+    schedule_notebook_save(background_tasks, uuid=uuid)
     return Response(status_code=status.HTTP_204_NO_CONTENT)
 
 
@@ -201,7 +208,7 @@ async def delete_notebook_cell(
     uuid: UUID, cell_uuid: UUID, background_tasks: BackgroundTasks
 ):
     manager.delete_cell(uuid, cell_uuid)
-    background_tasks.add_task(manager.save_notebook, uuid=uuid)
+    schedule_notebook_save(background_tasks, uuid=uuid)
     return Response(status_code=status.HTTP_204_NO_CONTENT)
 
 
@@ -307,6 +314,19 @@ async def get_notebook_vars(uuid: UUID):
     return KernelVariablesWrapper(vars=vars)
 
 
+@router.get(
+    "/{uuid}/vars/{var_name}",
+    dependencies=[Depends(write_access)],
+    response_model=KernelVariableWrapper,
+)
+async def get_notebook_single_var(uuid: UUID, var_name: str):
+    """Get details on a single var defined in the kernel."""
+    single_var = await manager.get_kernel_var_details(uuid, var_name)
+    if single_var is None:
+        raise HTTPException(404, f"Variable {var_name} not found")
+    return KernelVariableWrapper(single_var=single_var)
+
+
 @router.patch(
     "/{uuid}/cells/{cell_uuid}",
     response_model=CellWrapper,
@@ -319,7 +339,7 @@ async def patch_notebook_cell(
     background_tasks: BackgroundTasks,
 ) -> Any:
     cell = manager.update_cell(uuid, cell_uuid, cell_update)
-    background_tasks.add_task(manager.save_notebook, uuid=uuid)
+    schedule_notebook_save(background_tasks, uuid=uuid)
     return CellWrapper(cell=cell)
 
 
@@ -336,7 +356,7 @@ async def patch_and_execute_notebook_cell(
 ):
     cell = manager.update_cell(uuid, cell_uuid, cell_update)
     msg_id = await manager.execute(uuid, cell_uuid)
-    background_tasks.add_task(manager.save_notebook, uuid=uuid)
+    schedule_notebook_save(background_tasks, uuid=uuid)
     return CellKernelMessageWrapper(
         cell=cell, kernel_message=KernelMessage(message_id=msg_id),
     )
@@ -384,6 +404,7 @@ def format_file_path(
         "directory": str(filedir),
         "filename": file_path_obj.stem,
         "relative_path": str(relative_path),
+        "absolute_path": str(filepath),
     }
 
 
@@ -443,7 +464,7 @@ async def upload_and_open_notebook(
 
 
 @router.get(
-    "/open/{filepath:path}",
+    "/open/",
     status_code=status.HTTP_201_CREATED,
     dependencies=[Depends(write_access)],
     response_model=NotebookWrapper,
@@ -507,7 +528,7 @@ async def save_notebook_as(
     manager.save_notebook_as(
         uuid, path_info["directory"], path_info["filename"]
     )
-    return NotebookPath(path=path_info["relative_path"])
+    return NotebookPath(path=path_info["absolute_path"])
 
 
 @router.patch(
@@ -526,7 +547,7 @@ async def rename_notebook(
     manager.save_notebook_as(
         uuid, path_info["directory"], path_info["filename"], delete_old=True
     )
-    return NotebookPath(path=path_info["relative_path"])
+    return NotebookPath(path=path_info["absolute_path"])
 
 
 @router.websocket("/{uuid}/ws/stream")
@@ -802,4 +823,15 @@ async def websocket_cells(
 
     signal(KERNEL_RESTARTED).connect(receive_kernel_restarted)
 
+    async def receive_kernel_interrupted(
+        sender: Any, notebook_id: UUID, **kwargs,
+    ):
+        logger.debug("receiving KERNEL_INTERRUPTED")
+        if notebook_id == uuid:
+            wrapper_dict = {"kernel_interrupted": {"interrupted": True}}
+            wrapper_string = json.dumps(wrapper_dict, cls=D1Encoder)
+            cell_update_queue.append(wrapper_string)
+
+    signal(KERNEL_INTERRUPTED).connect(receive_kernel_interrupted)
+
     await websocket_poll(websocket, cell_update_queue)
