@@ -17,10 +17,11 @@ from jupyter_d1.signals import (CELL_ADDED, CELL_DELETED, CELL_EXECUTION_INPUT,
                                 CELL_EXECUTION_REPLY, CELL_EXECUTION_REQUEST,
                                 CELL_STREAM, CELL_UPDATE, COMPLETE_REPLY,
                                 CONTROL_CHANNEL, HB_CHANNEL, HISTORY_REPLY,
-                                IOPUB_CHANNEL, KERNEL_RESTARTED,
-                                METADATA_UPDATE, NOTEBOOKS_UPDATED,
-                                PAYLOAD_PAGE, SCRATCH_UPDATE, SHELL_CHANNEL,
-                                STDIN_CHANNEL, VARS_UPDATE)
+                                IOPUB_CHANNEL, KERNEL_INTERRUPTED,
+                                KERNEL_RESTARTED, METADATA_UPDATE,
+                                NOTEBOOKS_UPDATED, PAYLOAD_PAGE,
+                                SCRATCH_UPDATE, SHELL_CHANNEL, STDIN_CHANNEL,
+                                VAR_DETAILS, VARS_UPDATE)
 
 from ..d1_response import D1Encoder, d1_pairs_hook
 from ..kernels import VarsManager, WorkDirManager, get_kernel_definition
@@ -65,6 +66,7 @@ class CommandType(Enum):
     CELL_EXECUTION = auto()
     SCRATCH_EXECUTION = auto()
     VARS_REQUEST = auto()
+    VAR_DETAIL_REQUEST = auto()
     UPDATE_WORKDIR_REQUEST = auto()
     CHANGE_WORKDIR_REQUEST = auto()
     COMPLETE_REQUEST = auto()
@@ -186,7 +188,7 @@ class NotebookManager:
             raise NBMException(NBMErrorCode.notebook_not_found, uuid)
         old_path = nb.node.metadata.jupyter_d1.path
         self.validator.validate(
-            nb.node, directory, filename, new_notebook_uuid=True
+            nb.node, directory, filename,
         )
         self.save_notebook(nb.node)
 
@@ -416,6 +418,7 @@ class NotebookManager:
 
     async def interrupt_kernel(self, notebook_id: UUID):
         await self.kmanager.interrupt_kernel(notebook_id)
+        signal(KERNEL_INTERRUPTED).send(notebook_id=notebook_id,)
 
     async def perform_restart(self, notebook_id: UUID):
         await self.kmanager.restart_kernel(notebook_id)
@@ -492,6 +495,46 @@ class NotebookManager:
             CommandType.VARS_REQUEST, uuid
         )
 
+    async def get_kernel_var_details(
+        self, nb_id: UUID, var_name: str
+    ) -> Optional[KernelVariable]:
+        notebook = self.get_notebook(nb_id)
+        uuid = notebook.node.metadata.jupyter_d1.uuid
+        vars_manager = notebook.vars_manager
+        if vars_manager is None:
+            return None
+        code = vars_manager.get_single_var_code(var_name)
+        msg_id = await self.kmanager.execute(uuid, code, store_history=False)
+        self.msg_to_command[msg_id] = NotebookCommand(
+            CommandType.VAR_DETAIL_REQUEST, uuid, var_name=var_name
+        )
+        single_var_details: Optional[KernelVariable] = None
+        waiting = True
+
+        def receive_var_details(
+            sender: Any,
+            notebook_id: UUID,
+            var_name_inc: str,
+            single_var: KernelVariable,
+            **kwargs,
+        ):
+            if nb_id == notebook_id and var_name_inc == var_name:
+                nonlocal single_var_details
+                nonlocal waiting
+                single_var_details = single_var
+                waiting = False
+
+        signal(VAR_DETAILS).connect(receive_var_details)
+
+        count = 0
+        sleep_dur = 0.1
+        # Wait ~30 seconds
+        while waiting and count < (30 / sleep_dur):
+            await asyncio.sleep(sleep_dur)
+            count += 1
+
+        return single_var_details
+
     async def update_kernel_workdir(self, notebook_id: UUID):
         notebook = self.get_notebook(notebook_id)
         uuid = notebook.node.metadata.jupyter_d1.uuid
@@ -916,6 +959,7 @@ class NotebookManager:
                 suppress_stream_commands = [
                     CommandType.UPDATE_WORKDIR_REQUEST,
                     CommandType.VARS_REQUEST,
+                    CommandType.VAR_DETAIL_REQUEST,
                     CommandType.UPDATE_WORKDIR_REQUEST,
                 ]
                 if command.type not in suppress_stream_commands:
@@ -945,6 +989,18 @@ class NotebookManager:
                     and vars_manager is not None
                 ):
                     vars_manager.parse_output(output)
+            elif command.type == CommandType.VAR_DETAIL_REQUEST:
+                vars_manager = self.get_vars_manager(nb_id)
+                if (
+                    output.output_type in ("stream", "display_data")
+                    and vars_manager is not None
+                ):
+                    single_var = vars_manager.parse_single_var_response(output)
+                    signal(VAR_DETAILS).send(
+                        notebook_id=nb_id,
+                        var_name_inc=command.extras.get("var_name", None),
+                        single_var=single_var,
+                    )
             elif command.type in (
                 CommandType.UPDATE_WORKDIR_REQUEST,
                 CommandType.CHANGE_WORKDIR_REQUEST,
@@ -997,6 +1053,10 @@ class NotebookManager:
                         vars = vars_manager.on_request_end()
                         signal(VARS_UPDATE).send(notebook_id=nb_id, vars=vars)
                 return
+            elif command.type == CommandType.VAR_DETAIL_REQUEST:
+                if execution_state == ExecutionState.idle:
+                    self.msg_to_command.pop(parent_id)
+                return
             elif command.type in (
                 CommandType.UPDATE_WORKDIR_REQUEST,
                 CommandType.CHANGE_WORKDIR_REQUEST,
